﻿Public Class FormMain
  Private tmpEncoder As New System.Text.ASCIIEncoding

  Private SDB As New ArrayList ' Array of Strings
  Private FilteredSDB As New ArrayList ' Array of Indexes into SDB

  Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    ' Get the last opened path
    Try
      Me.TextBox_Path.Text = GetSetting("SDB Editor", "Main", "Path").Trim
    Catch
    End Try
  End Sub

  Private Sub Button_Open_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_Open.Click
    ' INTERFACE: disable all elements
    Me.TextBox_Path.Enabled = False
    Me.Button_SelectFile.Enabled = False
    Me.Button_Open.Enabled = False
    Me.Button_Save.Enabled = False

    ' INTERFACE: clear the filter on open
    Me.TextBox_Filter.text = ""

    ' INTERFACE: clear the existing list with items
    SDB.Clear()
    Me.DataGridView1.RowCount = SDB.Count

    ' Store the last opened path (even on failure)
    Dim SDBPath As String = Me.TextBox_Path.Text.Trim
    Try
      SaveSetting("SDB Editor", "Main", "Path", SDBPath)
    Catch
    End Try

    Try
      Dim Stream As IO.FileStream
      Stream = New IO.FileStream(SDBPath, IO.FileMode.Open, IO.FileAccess.Read, IO.FileShare.Read)
      Try
        ' Read the string and add it
        Dim StreamReader As New IO.StreamReader(Stream, tmpEncoder, False)
        Do While Not StreamReader.EndOfStream
          Dim StringContent As String = StreamReader.ReadLine

          SDB.Add(StringContent)
          FilteredSDB.Add(SDB.Count - 1)
        Loop

        ' 
        Me.DataGridView1.RowCount = FilteredSDB.Count
        DataGridView1.AutoResizeColumn(0, DataGridViewAutoSizeColumnMode.ColumnHeader)
        DataGridView1.AutoResizeColumn(1, DataGridViewAutoSizeColumnMode.AllCells)

        ' INTERFACE: we can open other files and we can save data
        Me.TextBox_Path.Enabled = True
        Me.Button_SelectFile.Enabled = True
        Me.Button_Open.Enabled = True
        Me.Button_Save.Enabled = True

        ' Success, so close the stream
        Stream.Close()
      Catch ex As Exception
        ' Close the stream even on error
        Try
          Stream.Close()
        Catch
        End Try

        ' Forward the error
        Throw ex
      End Try
    Catch ex As Exception
      ' INTERFACE: display the error message
      MsgBox(ex.Message)

      ' INTERFACE: we can open files again
      Me.TextBox_Path.Enabled = True
      Me.Button_SelectFile.Enabled = True
      Me.Button_Open.Enabled = True
    End Try
  End Sub

  Private Sub Button_Save_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_Save.Click
    ' Store the last opened path (even on failure)
    Dim SDBPath As String = Me.TextBox_Path.Text.Trim
    Try
      SaveSetting("SDB Editor", "Main", "Path", SDBPath)
    Catch
    End Try

    Try
      Dim Stream As IO.FileStream
      Stream = New IO.FileStream(SDBPath, IO.FileMode.Create, IO.FileAccess.Write, IO.FileShare.Read)
      Try
        Try
          ' Write all strings
          Dim Streamwriter As New IO.StreamWriter(Stream, tmpEncoder)
          For Each s As String In SDB
            Streamwriter.WriteLine(s)
          Next

          ' Success, so close the stream
          Stream.Close()
        Catch ex As Exception
          ' An error occurred while reading the file
          Try
            Stream.Close()
          Catch
          End Try
          Stream = Nothing

          ' INTERFACE: display the error message
          MsgBox(ex.Message)
        End Try
      Catch ex As Exception
        ' Close the stream even on error
        Try
          Stream.Close()
        Catch
        End Try

        ' Forward the error
        Throw ex
      End Try
    Catch ex As Exception
      ' INTERFACE: display the error message
      MsgBox(ex.Message)
    End Try
  End Sub

  Private Sub Button_SelectFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_SelectFile.Click
    Try
      ' Use a Open File Dialog to get a new file name
      Dim GetFile As New OpenFileDialog
      GetFile.CheckFileExists = True
      GetFile.DefaultExt = "TXT"
      GetFile.Filter = "TXT files (*.TXT)|*.TXT|All Files (*.*)|*.*"
      GetFile.FilterIndex = 0
      GetFile.FileName = Me.TextBox_Path.Text.Trim
      If GetFile.ShowDialog() = Windows.Forms.DialogResult.OK Then
        Me.TextBox_Path.Text = GetFile.FileName
      End If
    Catch ex As Exception
      ' Error!
      MsgBox(ex.Message)
    End Try
  End Sub

  Private Sub DataGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellValueEventArgs) Handles DataGridView1.CellValueNeeded
    ' Speaks for itself... 
    ' The DataGridView is in virtual mode so we need to give the content ourselves
    Try
      Select Case e.ColumnIndex
        Case 0
          e.Value = CType(FilteredSDB(e.RowIndex), Integer).ToString("X4")
        Case 1
          e.Value = CType(SDB(FilteredSDB(e.RowIndex)), String)
      End Select
    Catch
    End Try
  End Sub

  Private Sub DataGridView1_CellValuePushed(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellValueEventArgs) Handles DataGridView1.CellValuePushed
    Try
      ' Speaks for itself... 
      ' The DataGridView is in virtual mode so we need to set the content outselves
      If e.ColumnIndex = 1 Then
        SDB(FilteredSDB(e.RowIndex)) = e.Value
      End If
    Catch
    End Try
  End Sub

  Private Sub TextBox_Filter_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBox_Filter.TextChanged
    ' Clear the filter
    FilteredSDB.Clear()

    '
    Dim find As String = Me.TextBox_Filter.Text.ToLowerInvariant

    ' Add the index of each filtered item
    For i As Integer = 0 To SDB.Count - 1
      Dim s As String = SDB(i)
      If s.ToLowerInvariant.IndexOf(find) >= 0 Then
        FilteredSDB.Add(i)
      End If
    Next

    ' Update the DataGridView
    Me.DataGridView1.RowCount = FilteredSDB.Count
    Me.DataGridView1.Refresh()
  End Sub
End Class
